Attribute VB_Name = "mTitan"
Option Explicit

' APPERROR: Reports application errors and terminates application properly
Public Sub AppError(nNumber As Long, sText As String, sSource As String)

    ' Enable error handling
    On Error Resume Next
    
    ' Cleanup
    Call AppTerminate
    
    ' Display error
    MsgBox "ERROR: " & IIf(InStr(1, UCase(sText), "AUTOM") > 0, "DirectX reports '" & GetDXError(nNumber) & "'", " Application reports '" & sText & "'") & vbCrLf & "SOURCE: " & sSource, vbCritical + vbOKOnly, "TITAN"
    
    ' Terminate program
    End
    
End Sub

Public Sub AppInitialize()

    ' Enable error handling
        On Error GoTo E_AppInitialize

    ' Setup local variables...
    
        Dim L_dDDSD As DDSURFACEDESC           ' Utility surface description
        Dim L_dDDSC As DDSCAPS                 ' Utility display capabilities description
        Dim L_oD3DIM As IDirect3D2             ' Utility Direct3DIM interface for retrieving drivers
        Dim L_dDDCK As DDCOLORKEY              ' Color key for applying to various surfaces
        
    ' Initialize DirectDraw interface instance ...
    
        ' Create DirectDraw instance
        DirectDrawCreate ByVal 0&, G_oDDInstance, Nothing

        ' Check instance existance, terminate if missing
        If G_oDDInstance Is Nothing Then
           AppError 0, "Could not create DirectDraw instance", "AppInitialize"
           Exit Sub
        End If
         
        ' Set cooperation mode of DirectX
        
        G_oDDInstance.SetCooperativeLevel fTitan.hWnd, DDSCL_EXCLUSIVE Or DDSCL_FULLSCREEN
        
        ' Set display mode
        G_oDDInstance.SetDisplayMode 640, 480, 16
           
    ' Initialize primary surface ...
    
        ' Initialize primary surface description
        With L_dDDSD
            ' Get Structure size
            .dwSize = Len(L_dDDSD)
            ' Structure uses Surface Caps and count of BackBuffers
            .dwFlags = DDSD_CAPS Or DDSD_BACKBUFFERCOUNT
            ' Structure describes a flippable (buffered) surface
            .DDSCAPS.dwCaps = DDSCAPS_PRIMARYSURFACE Or DDSCAPS_FLIP Or DDSCAPS_COMPLEX Or DDSCAPS_SYSTEMMEMORY
            ' Structure uses one BackBuffer
            .dwBackBufferCount = 1
         End With
    
        ' Create primary surface from structure
        G_oDDInstance.CreateSurface L_dDDSD, G_oDDPrimary, Nothing
    
        ' Check primary existance, terminate if missing
        If G_oDDPrimary Is Nothing Then
           AppError 0, "Could not create primary surface", "AppInitialize"
           Exit Sub
        End If
    
    ' Initialize backbuffer from primary ...
    
        ' Set surface description to backbuffer creation
        L_dDDSD.dwFlags = DDSD_CAPS
        L_dDDSD.DDSCAPS.dwCaps = DDSCAPS_BACKBUFFER
        
        ' Create backbuffer from frontbuffer
        G_oDDPrimary.GetAttachedSurface L_dDDSD.DDSCAPS, G_oDDBackbuffer
        
        ' Check backbuffer existance, terminate if missing
        If G_oDDBackbuffer Is Nothing Then
           AppError 0, "Could not create backbuffer", "AppInitialize"
           Exit Sub
        End If
        
        ' Create DirectX surface for Direct3D to dra on
        Set G_oDDD3DBuffer = MakeDXSurface(200, 200, True)
        
    ' Initialize sound system ...
    
        ' Create an instance of DirectSound interface
        DirectSoundCreate ByVal 0&, G_oDSInstance, Nothing
            
        ' Check instance existance, terminate if missing
        If G_oDSInstance Is Nothing Then
           AppError 0, "Could not create DirectSound instance", "AppInitialize"
           Exit Sub
        End If
                    
        ' Set sound system cooperative level
        G_oDSInstance.SetCooperativeLevel fTitan.hWnd, DSSCL_NORMAL
        
    ' Initialize background music
        Set G_oDSBufferMusic = LoadWaveIntoDSB(App.Path + "\music.wav")
        G_oDSBufferMusic.Play ByVal 0&, ByVal 0&, DSBPLAY_LOOPING
        
    ' Initialize Direct3DRM interface instance ...
    
        ' Create Direct3DRM instance
        Direct3DRMCreate G_oD3DInstance
    
        ' Check instance existance, terminate if missing
        If G_oD3DInstance Is Nothing Then
           AppError 0, "Could not create D3DRM instance", "AppInitialize"
           Exit Sub
        End If
    
    ' Initialize Direct3DRM driver ...
    
        ' Get a Direct3D immediate object from the existing DirectDraw object
        Set L_oD3DIM = G_oDDInstance
    
        ' Set error handler to local for enumeration only
        On Error Resume Next
        
        ' Start the callback that does the driver enumeration
        L_oD3DIM.EnumDevices AddressOf EnumDeviceCallback, 0
    
        ' Catch any error resulting from the enumeration and terminate
        If Err.Number > 0 Then
           AppError Err.Number, Err.Description, "AppInitialize"
           Exit Sub
        End If
    
        ' Reset error handler to default
        On Error GoTo E_AppInitialize
        
        ' Reset Direct3D immediate object
        Set L_oD3DIM = Nothing
        
        ' Check if a convenient device driver has been found, terminate if no driver available
        If Not G_bD3DDriverPresent Then
           AppError 0, "Could not detect Direct3D device driver", "AppInitialize"
           Exit Sub
        End If
        
    ' Initialize D3DRM display system
    
        ' Create a D3DRM device from the 3D buffer
        G_oD3DInstance.CreateDeviceFromSurface G_dD3DDriver.GUID, G_oDDInstance, G_oDDD3DBuffer, G_oD3DDevice
        
        ' Check device existance, terminate if missing
        If G_oD3DDevice Is Nothing Then
           AppError 0, "Could not create D3DRM device", "AppInitialize"
           Exit Sub
        End If
    
        ' Set D3DRM device quality
        G_oD3DDevice.SetQuality D3DRMLIGHT_ON Or D3DRMFILL_SOLID Or D3DRMSHADE_PHONG
        
        ' Create the master frame containing all other frames
        G_oD3DInstance.CreateFrame Nothing, G_oD3DMasterFrame
        
        ' Create the camera frame containing the primary camera
        G_oD3DInstance.CreateFrame G_oD3DMasterFrame, G_oD3DCameraFrame
        
        ' Create a D3D viewport from the device, using the camera frame for output
        G_oD3DInstance.CreateViewport G_oD3DDevice, G_oD3DCameraFrame, 0, 0, 200, 200, G_oD3DViewport
            
        ' Check viewport existance, terminate if missing
        If G_oD3DViewport Is Nothing Then
           AppError 0, "Could not create D3DRM viewport", "AppInitialize"
           Exit Sub
        End If
        
        ' Error handling ...
        
        Exit Sub

E_AppInitialize:

        AppError Err.Number, Err.Description, "AppInitialize"
        
End Sub

Public Sub AppLoop()

    ' Enable error handling
        On Error GoTo E_AppLoop

    ' Setup local variables...
        Dim L_nNextFrameTime As Long        ' Timer used to time frames to a minimum duration
        Dim L_nFrameCount As Long           ' Frame counter used for calculating average framerate
        Dim L_nNextSecond As Long           ' TimeGetTime value above which next second begins
        Dim L_nCurrentTime As Long          ' Time at start of frame, to avoid multiple calls of TimeGetTime
        
        Dim L_dRenderArea As RECT           ' Rectangle to describe render area for blitting
        Dim L_dDDBLTFX As DDBLTFX           ' Holds F/X settings for blitting
        
    ' Preparations for master loop
    
        ' Prepare BLTFX structure for color fill blit to clear backbuffer
        With L_dDDBLTFX
            .dwSize = Len(L_dDDBLTFX)
            .dwFillColor = 0
        End With
        
    ' Master loop controlling application behavior...
        
        Do
            
            ' Do frame timing and statistics ...
            
                ' Increase global frame counter
                G_nFrameCount = G_nFrameCount + 1
                
                ' Get frame start time
                L_nCurrentTime = timeGetTime
                
                ' Increase frame count for avg frametime calculation
                L_nFrameCount = L_nFrameCount + 1
                
                ' Protocol frame time: Count frames and write out average frame count every second
                If L_nNextSecond < L_nCurrentTime Then
                    G_nFrameAvg = (G_nFrameAvg + L_nFrameCount) / 2
                    L_nNextSecond = L_nCurrentTime + 1000
                    L_nFrameCount = 0
                End If
            
                ' Prepare timing: Set next frame time to current time plus minimum frame duration
                L_nNextFrameTime = L_nCurrentTime + 50
            
                ' Query user input
                DoEvents
                        
            ' Clear backbuffer ...
                
                ' FX-Blit filling background with black
                With L_dRenderArea
                    .Top = 0
                    .Left = 0
                    .Bottom = 480
                    .Right = 640
                End With
                G_oDDBackbuffer.Blt L_dRenderArea, ByVal Nothing, ByVal 0&, DDBLT_COLORFILL, L_dDDBLTFX
            
            ' Your drawing code goes here ...
            
                ' ...
                ' Example 1: If you are using D3DRM, you have to blit the Direct3DRM buffer to the backbuffer
                ' Example 2: If you are using DirectDraw, you have to blit your sprites to the backbuffer

            
            ' Flip DirectX buffers...
                G_oDDPrimary.Flip Nothing, 0
                
            ' Do timing: Loop until minimum time per frame reached ...
            Do
            Loop Until timeGetTime > L_nNextFrameTime
        
        Loop
        
    ' Error handling ...
    
        Exit Sub

E_AppLoop:

        ' Resume to ignore the weird math errors Direct3DRM reports from time to time
        Resume Next

End Sub

Public Sub AppTerminate()

    ' Enable error handling...
        On Error GoTo E_AppTerminate

    ' Setup local variables ...
        Dim L_nRun As Integer             ' Variable to run through various array data
    
    ' Return control from DirectX to windows ...

        ' Restore old resolution and depth
        G_oDDInstance.RestoreDisplayMode
    
        ' Return control to windows
        G_oDDInstance.SetCooperativeLevel fTitan.hWnd, DDSCL_NORMAL
        
    ' Reset DirectX objects ...
        
        
        ' D3DRM display system ...
        
            Set G_oD3DCameraFrame = Nothing
            Set G_oD3DMasterFrame = Nothing
            Set G_oD3DViewport = Nothing
            Set G_oD3DDevice = Nothing
            
        ' DD Display system ...
        
            Set G_oDDBackbuffer = Nothing
            Set G_oDDPrimary = Nothing
            Set G_oDDD3DBuffer = Nothing
                
        ' DirectSound ...
        
            G_oDSBufferMusic.Stop
            Set G_oDSBufferMusic = Nothing
        
        ' DirectX interfaces ...
        
            Set G_oDSInstance = Nothing
            Set G_oDDInstance = Nothing
            Set G_oD3DInstance = Nothing
        
    ' Error handling ...
        
        Exit Sub

E_AppTerminate:

        ' Resume to ensure that all objects available are cleaned up
        Resume Next

End Sub

